//
//  FileTableViewDataSource.m
//  TNEFsEnough3
//
//  Created by Josh Jacob on 7/14/11.
//  Copyright 2011 Josh Jacob. All rights reserved.
//

#import "FileTableViewDataSource.h"
#import "FileTableItem.h"

@implementation FileTableViewDataSource

@synthesize fileArray = _fileArray;

- (id) init 
{
    self = [super init];
    if (self != nil)
    {
        _fileArray = [[NSMutableArray alloc] initWithCapacity: 10];
    }
    return self;
}

- (void) dealloc
{
    [self.fileArray removeAllObjects];
    self.fileArray = nil;
}

- (id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)aTableColumn row:(NSInteger)rowIndex
{
    NSParameterAssert(rowIndex >= 0 && rowIndex < [self.fileArray count]);
    
    FileTableItem * file = [self.fileArray objectAtIndex:rowIndex];
    
    if ([[aTableColumn identifier] isEqualToString:@"Size"]) {
        return [file stringFromFileSize];
    } else if ([[aTableColumn identifier] isEqualToString:@"File Name"]) {
        return file.fileName;
    } else {
        return nil;
    }
}

- (NSInteger)numberOfRowsInTableView:(NSTableView *)aTableView
{
    return [self.fileArray count];
}


- (BOOL)tableView:(NSTableView *)aTableView writeRowsWithIndexes:(NSIndexSet *)rowIndexes toPasteboard:(NSPasteboard *)pboard
{
    NSLog(@"writeRowsWithIndexes");
    
    [pboard declareTypes:[NSArray arrayWithObjects:NSFilesPromisePboardType, nil] owner:self];
    
    NSMutableArray *filenameExtensions = [NSMutableArray array];
    NSArray *draggedFilenames = [self.fileArray objectsAtIndexes:rowIndexes];
    for (FileTableItem *fileTableItem in draggedFilenames) {
        NSString *filenameExtension = [fileTableItem.fileName pathExtension];
        // if (![filenameExtension isEqualToString:@""]) {
            [filenameExtensions addObject:filenameExtension];
        // }
    }
    [pboard setPropertyList:filenameExtensions forType:NSFilesPromisePboardType];
    return YES;
}

- (NSArray *)tableView:(NSTableView *)aTableView namesOfPromisedFilesDroppedAtDestination:(NSURL *)dropDestination forDraggedRowsWithIndexes:(NSIndexSet *)indexSet {
    
    NSArray *askedForItems = [self.fileArray objectsAtIndexes:indexSet];
    NSMutableArray *draggedFilenames = [NSMutableArray array];
    
    for (FileTableItem *fileTableItem in askedForItems) {
        
        NSString *filenameExtension = [fileTableItem.fileName pathExtension];
        NSString *filepath  = [FileTableViewDataSource GenerateUniqueFileNameAtPath: [dropDestination path] WithBase:[fileTableItem.fileName stringByDeletingPathExtension] Extension:filenameExtension];
        
        [draggedFilenames addObject:[filepath lastPathComponent]];
        
        Attachment * attachment = fileTableItem.attachment;
        
        NSData *fileData = [[NSData alloc] initWithBytes:attachment->FileData.data length:attachment->FileData.size];
        
        NSError *error = nil;
        if (![fileData writeToURL:[NSURL fileURLWithPath:filepath] options:0 error:&error]) {
            [NSApp presentError:error];
            
        }
        
        [fileData release];
    }
    return draggedFilenames;
}

+ (NSString *) GenerateUniqueFileNameAtPath:(NSString *) path WithBase: (NSString *)basename Extension: (NSString *)extension;
{
    NSString *filename = [extension length] > 0 ? [NSString stringWithFormat:@"%@.%@", basename, extension] : [NSString stringWithFormat:@"%@", basename];
    NSString *result = [path stringByAppendingPathComponent:filename];
    NSInteger i = 1;
    while ([[NSFileManager defaultManager] fileExistsAtPath:result]) {
        filename = [extension length] > 0 ? [NSString stringWithFormat:@"%@ %ld.%@", basename, (long)i, extension] : [NSString stringWithFormat:@"%@ %ld", basename, (long)i];
        result = [path stringByAppendingPathComponent:filename];
        i++;
    }    
    return result;
}

@end
